@@ -163,9 +163,11 @@ pub mod prelude {
     use super::*;
 
     pub use super::BigBrainPlugin;
-    pub use actions::{ActionBuilder, ActionState, Steps, Concurrently};
+    pub use actions::{ActionBuilder, ActionState, Concurrently, Steps};
     pub use pickers::{FirstToScore, Picker};
-    pub use scorers::{AllOrNothing, FixedScore, Score, ScorerBuilder, SumOfScorers, WinningScorer};
+    pub use scorers::{
+        AllOrNothing, FixedScore, Score, ScorerBuilder, SumOfScorers, WinningScorer,
+    };
     pub use thinker::{Actor, Thinker, ThinkerBuilder};
 }
 
@@ -191,16 +193,31 @@ pub struct BigBrainPlugin;
 
 impl Plugin for BigBrainPlugin {
     fn build(&self, app: &mut AppBuilder) {
-        app.add_system(thinker::thinker_system.system());
-        app.add_system(thinker::thinker_component_attach_system.system());
-        app.add_system(thinker::thinker_component_detach_system.system());
-        app.add_system(thinker::actor_gone_cleanup.system());
-        app.add_system(actions::steps_system.system());
-        app.add_system(actions::concurrent_system.system());
-        app.add_system(scorers::fixed_score_system.system());
-        app.add_system(scorers::all_or_nothing_system.system());
-        app.add_system(scorers::sum_of_scorers_system.system());
-        app.add_system(scorers::winning_scorer_system.system());
-        app.add_system(scorers::evaluating_scorer_system.system());
+        use CoreStage::*;
+        app.add_system_set_to_stage(
+            First,
+            SystemSet::new()
+                .with_system(scorers::fixed_score_system.system())
+                .with_system(scorers::all_or_nothing_system.system())
+                .with_system(scorers::sum_of_scorers_system.system())
+                .with_system(scorers::winning_scorer_system.system())
+                .with_system(scorers::evaluating_scorer_system.system())
+                .label("scorers"),
+        );
+        app.add_system_to_stage(First, thinker::thinker_system.system().after("scorers"));
+
+        app.add_system_set_to_stage(
+            PreUpdate,
+            SystemSet::new()
+                .with_system(actions::steps_system.system())
+                .with_system(actions::concurrent_system.system())
+                .label("aggregate-actions"),
+        );
+
+        // run your actions in PreUpdate after aggregate-actions or in a later stage
+
+        app.add_system_to_stage(Last, thinker::thinker_component_attach_system.system());
+        app.add_system_to_stage(Last, thinker::thinker_component_detach_system.system());
+        app.add_system_to_stage(Last, thinker::actor_gone_cleanup.system());
     }
 }
